/**
 * @file
 * Site preview.
 */

(function ($, window, Drupal, once, drupalSettings) {
  $.fn.dialogToggleOnlyWhenCollapsed = () => {
    const dialogWidth = jQuery('#drupal-off-canvas').dialog('option', 'width');
    if (dialogWidth <= 440) {
      $.fn.dialogToggle();
    }
  };

  $.fn.dialogToggle = () => {
    const dialogWidth = jQuery('#drupal-off-canvas').dialog('option', 'width');
    const state = dialogWidth <= 440 ? 'collapsed' : 'expanded';
    const width = state === 'collapsed' ? '1200' : '440';
    const label = state === 'collapsed' ? '>>' : '<<';

    jQuery('#dialog-toggle').html(label);
    jQuery('#drupal-off-canvas').dialog('option', {
      width,
      resizable: false,
    });
  };

  $.fn.visualEditorOpenDialog = (url, dialog) => {
    const ajaxSettings = {
      url,
      dialogRenderer: 'off_canvas',
      dialogType: 'dialog',
      dialog,
    };

    Drupal.ajax(ajaxSettings).execute();
  };

  $.fn.visualEditorReload = (storage, id) => {
    if (storage === 'node' && id) {
      // Reload node after save to refresh form state and prevent error:
      // The content has either been modified by another user,
      // or you have already submitted modifications.
      // As a result, your changes cannot be saved.
      // Reload Drupal page to refresh form state.
      window.location.href = `/node/${id}`;
    }
  };

  Drupal.behaviors.visualEditorLoad = {
    attach(context) {
      const {
        openLoad,
        isPreview = false,
        node,
      } = drupalSettings.visual_editor;
      const { selector } = drupalSettings.decoupled_preview_iframe;
      // Attach dialog toggle event
      jQuery('#dialog-toggle').bind('click', () => {
        $.fn.dialogToggle();
      });
      once('visual_editor', selector, context).forEach(() => {
        if (openLoad && node) {
          $.fn.visualEditorOpenDialog(
            `/visual_editor/form/node/${node}/edit?preview=${isPreview}`,
            // @todo: make width configurable
            {
              width: 440,
              resizable: false,
            },
          );
        }
      });
    },
  };

  Drupal.behaviors.visualEditorListener = {
    attach() {
      window.addEventListener(
        'message',
        (event) => {
          const { wrapper } = drupalSettings.visual_editor;
          const { data } = event;
          if (data.type === 'VISUAL_EDITOR_COMPONENT') {
            // Exapnd dialog if collapsed
            $.fn.dialogToggleOnlyWhenCollapsed();

            const { uuid } = data;
            const uuidInput = $(`input[data-visual-editor-uuid="${uuid}"]`);
            const inputId = uuidInput.attr('id');
            const divSelector = `edit-${inputId.substring(
              0,
              inputId.indexOf('-edit--'),
            )}`;
            const divId = $(`div[data-drupal-selector="${divSelector}"]`).attr(
              'id',
            );

            if (
              wrapper.type &&
              wrapper.mode &&
              wrapper.selector &&
              wrapper.position
            ) {
              if (wrapper.type === 'tab' && Number(wrapper.position) > 0) {
                const wrapperSelector = `${wrapper.selector} > .${
                  wrapper.mode
                }-${wrapper.type}s > .${wrapper.mode}-${
                  wrapper.type
                }s-list > .${wrapper.mode}-${wrapper.type}-button-${
                  Number(wrapper.position) - 1
                }`;
                $(wrapperSelector).children(0).click();
              }
            }

            // Expand paragraph
            uuidInput.trigger('mousedown');
            // Scroll to paragraph
            document.getElementById(divId).scrollIntoView({
              behavior: 'smooth',
              block: 'start',
              inline: 'end',
            });
            event.stopImmediatePropagation();
          }
        },
        false,
      );
    },
  };
})(jQuery, window, Drupal, once, drupalSettings);
